

class RCatVal:
    obj_list = []

    def __init__(self, params):
        self.rsrc_catg_id = params.get('rsrc_catg_id').strip() if params.get('rsrc_catg_id') else None
        self.rsrc_catg_type_id = params.get('rsrc_catg_type_id').strip() if params.get('rsrc_catg_type_id') else None
        self.rsrc_catg_short_name = params.get('rsrc_catg_short_name').strip() if params.get('rsrc_catg_short_name') else None
        self.rsrc_catg_name = params.get('rsrc_catg_name').strip() if params.get('rsrc_catg_name') else None
        self.parent_rsrc_catg_id = params.get('parent_rsrc_catg_id').strip() if params.get('parent_rsrc_catg_id') else None
        RCatVal.obj_list.append(self)
    def get_id(self):
        return self.rsrc_catg_id

    @classmethod
    def find_by_id(cls, id):
        obj = list(filter(lambda x: x.rsrc_catg_id == id, cls.obj_list))
        if obj:
            return obj[0]
        return obj

    def __repr__(self):
        return self.rsrc_catg_name


'''
This file starts the process of reading and parsing xer files

'''
import csv
from xerparser import *


class Reader:

    current_table = ''
    current_headers = []
    # currencies = []
    # obs = []
    #
    # project = []
    # calendar = []
    # wbs = []
    # resource = []
    #
    # task = []

    def create_object(self, object_type, params):
        """

        Args:
            object_type:
            params:

        Returns:

        """
        if object_type.strip() == "CURRTYPE":
            obj = Currency(params)
            return obj
        elif object_type.strip() == "ROLES":
            obj = Role(params)
            return obj
        elif object_type.strip() == "ACCOUNT":
            self._accounts.add_account(params)

        elif object_type.strip() == "ROLERATE":
            obj = RoleRate(params)
            return obj
        elif object_type.strip() == "OBS":
            obj = OBS(params)
            return obj
        elif object_type.strip() == "RCATTYPE":
            obj = RCatType(params)
            return obj
        elif object_type.strip() == "UDFTYPE":
            obj = UDFType(params)
            return obj
        elif object_type.strip() == "RCATVAL":
            obj = RCatVal(params)
            return obj
        elif object_type.strip() == "PROJECT":
            self._projects.add_project(params)

        elif object_type.strip() == "CALENDAR":
            obj = Calendar(params)
            # print(params)
            return obj
        elif object_type.strip() == "SCHEDOPTIONS":
            obj = SchedOption(params)
            return obj
        elif object_type.strip() == "PROJWBS":
            self._wbss.add_wbs(params)
        elif object_type.strip() == "RSRC":
            self._resources.add_resource(params)
            obj = Resource(params)
            return obj
        elif object_type.strip() == "ACTVTYPE":
            obj = ActType(params)
            return obj
        elif object_type.strip() == "RSRCRATE":
            obj = ResourceRate(params)
            return obj
        elif object_type.strip() == "RSRCRCAT":
            obj = ResourceCat(params)
            return obj
        elif object_type.strip() == "TASK":
            self._tasks.add_task(params)
            #self.task.append(obj)
            # return obj
        elif object_type.strip() == "ACTVCODE":
            obj = ActivityCode(params)
            return obj
        elif object_type.strip() == "TASKPRED":
            # obj = TaskPred(params)
            self._predecessors.add_predecessor(params)
        elif object_type.strip() == "TASKRSRC":
            obj = TaskRsrc(params)
            return obj
        elif object_type.strip() == "TASKACTV":
            obj = TaskActv(params)
            return obj
        elif object_type.strip() == "UDFVALUE":
            obj = UDFValue(params)
            return obj

    def summary(self):
        print('Number of activities: ', self.tasks.count)
        print('Number of relationships: ', len(TaskPred.obj_list))

    def __init__(self, filename):
        file = open(filename, 'r')
        self._tasks = Tasks()
        self._predecessors = Predecessors()
        self._projects = Projects()
        self._wbss = WBSs()
        self._resources = Resources()
        self._accounts = Accounts()
        content = file.readlines()
        with open(filename) as tsvfile:
            stream = csv.reader(tsvfile, delimiter='\t')
            for row in stream:
                if row[0] =="%T":
                    current_table = row[1]
                elif row[0] == "%F":
                    current_headers = [r.strip() for r in row[1:]]
                elif row[0] == "%R":
                    zipped_record = dict(zip(current_headers, row[1:]))
                    self.create_object(current_table, zipped_record)
        # for line in content:
        #     line_lst = line.split('\t')
        #     if line_lst[0] == "%T":
        #         current_table = line_lst[1]
        #     elif line_lst[0] == "%R":
        #         self.create_object(current_table, line_lst[1:])





class ResourceCat:
    obj_list = []

    def __init__(self, params):
        self.rsrc_id = int(params.get('rsrc_id').strip()) if params.get('rsrc_id') else None
        self.rsrc_catg_type_id = int(params.get('rsrc_catg_type_id').strip()) if params.get('rsrc_catg_type_id') else None
        self.rsrc_catg_id = int(params.get('rsrc_catg_type_id').strip()) if params.get('rsrc_catg_type_id') else None
        ResourceCat.obj_list.append(self)

    def get_id(self):
        return self.rsrc_id

    def __repr__(self):
        return self.rsrc_id + ' has been assign category ' + self.rsrc_catg_id





class Role:
    obj_list = []

    def __init__(self, params):
        self.role_id = int(params.get('role_id')) if params.get('role_id') else None
        self.parent_role_id = int(params.get('parent_role_id')) if params.get('parent_role_id') else None
        self.seq_num = int(params.get('seq_num')) if params.get('seq_num') else None
        self.role_name = params.get('role_name') if params.get('role_name') else None
        self.role_short_name = params.get('role_short_name') if params.get('role_short_name') else None
        self.pobs_id = params.get('pobs_id') if params.get('pobs_id') else None
        self.def_cost_qty_link_flag = params.get('def_cost_qty_link_flag') if params.get('def_cost_qty_link_flag') else None
        self.cost_qty_type = params.get('cost_qty_type') if params.get('cost_qty_type') else None
        self.role_descr = params.get('role_descr') if params.get('role_descr') else None
        self.last_checksum = params.get('role_descr') if params.get('role_descr') else None

        Role.obj_list.append(self)

    def __repr__(self):
        return self.role_name



class TaskActv:
    obj_list = []

    def __init__(self, params):
        self.task_id = int(params.get('task_id').strip()) if params.get('task_id') else None
        self.actv_code_type_id = params.get('actv_code_type_id').strip()
        self.actv_code_id = int(params.get('actv_code_id').strip()) if params.get('actv_code_id') else None
        self.proj_id = int(params.get('proj_id').strip()) if params.get('proj_id') else None

        TaskActv.obj_list.append(self)

    def get_id(self):
        return self.task_id

    @staticmethod
    def find_by_id(code_id, activity_code_dict):
        return {k: v for k, v in activity_code_dict.items() if v.actv_code_id == code_id}

    def __repr__(self):
        return self.task_id + '->' + self.actv_code_id


from xerparser.model.classes.activitycode import ActivityCode

class ActType:
    obj_list =[]

    def __init__(self, params):
        # Unique ID generated by the system.
        self.actv_code_type_id = int(params.get('actv_code_type_id').strip()) if params.get('actv_code_type_id') else None
        # The maximum number of characters allowed for values of this Activity Code.
        self.actv_short_len = float(params.get('actv_short_len').strip()) if params.get('actv_short_len') else None
        # Sequence number for sorting.
        self.seq_num = int(params.get('seq_num').strip()) if params.get('seq_num') else None
        # Each Activity Code has a list of possible values, any of which can be assigned to an activity. Activity
        # Codes allow you to classify and categorize activities.
        self.actv_code_type = params.get('actv_code_type').strip() if params.get('actv_code_type') else None
        # Identifies the project which owns this Activity Code (if this is a project-level Activity Code).
        self.proj_id = int(params.get('proj_id')) if params.get('proj_id') else None
        # EPS Id element that
        self.wbs_id = params.get('wbs_id').strip() if params.get('wbs_id') else None
        self.actv_code_type_scope = params.get('actv_code_type_scope').strip() if params.get('actv_code_type_scope') else None
        ActType.obj_list.append(self)

    def get_id(self):
        return self.actv_code_type_id

    def get_activity_codes(self):
        obj = list(filter(lambda x: x.actv_code_type_id == self.actv_code_type_id, ActivityCode.obj_list))
        return obj

    @classmethod
    def find_by_id(cls, id):
        """ Function to search list of activity code type by an ID

        Args:
            id: Unique ID generated by the system.

        Returns: ActType that matches the ID

        """
        obj = list(filter(lambda x: x.actv_code_type_id == id, cls.obj_list))
        if len(obj) > 0:
            return obj[0]
        else:
            obj = None
        return obj

    def __repr__(self):
        return self.actv_code_type




from xerparser.model.classes.account import Account

class Accounts:
    _accounts = []
    def __init__(self):
        self.index = 0

    def add_account(self, params):
        self._accounts.append(Account(params))

    @staticmethod
    def count():
        return len(Accounts.task_pred)

    def __iter__(self):
        return self

    def __next__(self):
        if self.index >= len(self.task_pred):
            raise StopIteration
        idx = self.index
        self.index += 1
        return self.task_pred[idx]



class SchedOption:
    obj_list = []

    def __init__(self, params):

        self.schedoptions_id = params.get('schedoptions_id').strip() if params.get('schedoptions_id') else None
        self.proj_id = params.get('proj_id').strip() if params.get('proj_id') else None
        self.sched_outer_depend_type = params.get('sched_outer_depend_type').strip() if params.get('sched_outer_depend_type') else None
        self.sched_open_critical_flag = params.get('sched_open_critical_flag').strip() if params.get('sched_open_critical_flag') else None
        self.sched_lag_early_start_flag = params.get('sched_lag_early_start_flag').strip() if params.get('sched_lag_early_start_flag') else None
        self.sched_retained_logic = params.get('sched_retained_logic').strip() if params.get('sched_retained_logic') else None
        self.sched_setplantoforecast = params.get('sched_setplantoforecast').strip() if params.get('sched_setplantoforecast') else None
        self.sched_float_type = params.get('sched_float_type').strip() if params.get('sched_float_type') else None
        self.sched_calendar_on_relationship_lag = params.get('sched_calendar_on_relationship_lag').strip() if params.get('sched_calendar_on_relationship_lag') else None
        self.sched_use_expect_end_flag = params.get('sched_use_expect_end_flag').strip() if params.get('sched_use_expect_end_flag') else None
        self.sched_progress_override = params.get('sched_progress_override').strip() if params.get('sched_progress_override') else None
        self.level_float_thrs_cnt = params.get('level_float_thrs_cnt').strip() if params.get('level_float_thrs_cnt') else None
        self.level_outer_assign_flag = params.get('level_outer_assign_flag').strip() if params.get('level_outer_assign_flag') else None
        self.level_outer_assign_priority = params.get('level_outer_assign_priority').strip() if params.get('level_outer_assign_priority') else None
        self.level_over_alloc_pct = params.get('level_over_alloc_pct').strip() if params.get('level_over_alloc_pct') else None
        self.level_within_float_flag = params.get('level_within_float_flag').strip() if params.get('level_within_float_flag') else None
        self.level_keep_sched_date_flag = params.get('level_keep_sched_date_flag').strip() if params.get('level_keep_sched_date_flag') else None
        self.level_all_rsrc_flag = params.get('level_all_rsrc_flag').strip() if params.get('level_all_rsrc_flag') else None
        self.sched_use_project_end_date_for_float = params.get('sched_use_project_end_date_for_float').strip() if params.get('sched_use_project_end_date_for_float') else None
        self.enable_multiple_longest_path_calc = params.get('enable_multiple_longest_path_calc').strip() if params.get('enable_multiple_longest_path_calc') else None
        self.limit_multiple_longest_path_calc = params.get('limit_multiple_longest_path_calc').strip() if params.get('limit_multiple_longest_path_calc') else None
        self.max_multiple_longest_path = params.get('max_multiple_longest_path').strip() if params.get('max_multiple_longest_path') else None
        self.use_total_float_multiple_longest_paths = params.get('use_total_float_multiple_longest_paths').strip() if params.get('use_total_float_multiple_longest_paths') else None
        self.key_activity_for_multiple_longest_paths = params.get('use_total_float_multiple_longest_paths').strip() if params.get('use_total_float_multiple_longest_paths') else None
        self.LevelPriorityList = params.get('LevelPriorityList').strip() if params.get('LevelPriorityList') else None
        SchedOption.obj_list.append(self)

    def get_id(self):
        return self.schedoptions_id

    def __repr__(self):
        return self.proj_id




class ActivityStatus:

    Complete = "TK_Complete"
    InProgress = "TK_Active"
    NotStarted = "TK_NotStart"


class ActivityType:

    Task = "TT_Task"
    StartMilestone = "TT_Mile"
    FinishMilestone = "TT_FinMile"
    LevelOfEffort = "TT_LOE"
    WBSSummary = "TT_WBS"


class RelationshipType:

    FF = "PR_FF"
    FS = "PR_FS"
    SS = "PR_SS"
    SF = "PR_SF"


class DurationTypes:

    FixedQuantity = "DT_FixedQty"
    FixedDurationAndUnits = "DT_FixedDrtn"
    FixedRate = "DT_FixedRate"
    FixedDuration = "DT_FixedDUR2"



from xerparser.model.classes.rsrc import Resource

class Resources:

    _rsrcs = []

    def __init__(self):
        self.index = 0

    def add_resource(self, params):
        rsrc = Resource(params)
        self._rsrcs.append(rsrc)

    @staticmethod
    def get_resource_by_id(id):
        rsrc = list(filter(lambda x: x.rsrc_id == id, Resources._rsrcs))
        if len(rsrc) > 0:
            rsrc = rsrc[0]
        else:
            rsrc = None
        return rsrc

    @staticmethod
    def get_parent(id):
        rsrc = list(filter(lambda x: x.rsrc_id == id, Resources._rsrcs))
        if len(rsrc) > 0:
            rsrc = rsrc[0]
            parent = Resources.get_resource_by_id(rsrc.parent_rsrc_id)
        else:
            rsrc = None
        return rsrc


    def __iter__(self):
        return self

    def __next__(self):
        if self.index >= len(self._rsrcs):
            raise StopIteration
        idx = self.index
        self.index +=1
        return self._rsrcs[idx]

    def _get_list(self):
        resor = []
        for res in Resources._rsrcs:
            resor.append((res.rsrc_id, res.parent_rsrc_id))
        return resor

    def build_tree(self):
        # pass 1: create nodes dictionary
        a = self._get_list()
        nodes = {}
        for i in a:
            id, parent_id = i
            nodes[id] = {id : self.get_resource_by_id(id)}
        # a = a[1:]
        # pass 2: create trees and parent-child relations
        forest = []
        for i in a:
            id, parent_id = i
            node = nodes[id]
            # either make the node a new tree or link it to its parent
            if parent_id == None or nodes.get(parent_id) == None:
                # start a new tree in the forest
                forest.append(node)
            else:
                # add new_node as child to parent
                parent = nodes.get(parent_id)

                if not 'children' in parent:
                    # ensure parent has a 'children' field
                    parent['children'] = []
                children = parent['children']
                children.append(node)
        return forest




class RCatType:
    obj_list = []

    def __init__(self, params):
        self.rsrc_catg_type_id = int(params.get('rsrc_catg_type_id').strip()) if params.get('rsrc_catg_type_id') else None
        self.seq_num = params.get('seq_num').strip() if params.get('seq_num') else None
        self.rsrc_catg_short_len = params.get('rsrc_catg_short_len').strip() if params.get('rsrc_catg_short_len') else None
        self.rsrc_catg_type = params.get('rsrc_catg_type').strip() if params.get('rsrc_catg_type') else None
        RCatType.obj_list.append(self)

    def get_id(self):
        return self.rsrc_catg_type_id

    @classmethod
    def find_by_id(cls, id):
        obj = list(filter(lambda x: x.rsrc_catg_type_id == id, cls.obj_list))
        if obj:
            return obj[0]
        return obj

    def __repr__(self):
        return self.rsrc_catg_type


from xerparser.model.classes.calendar import Calendar
from xerparser.model.classes.activitycode import ActivityCode
from xerparser.model.classes.acttype import ActType
from xerparser.model.classes.obs import OBS
from xerparser.model.classes.project import Project
from xerparser.model.classes.rcattype import RCatType
from xerparser.model.classes.rcatval import RCatVal
from xerparser.model.classes.rsrc import Resource
from xerparser.model.classes.rsrcrate import ResourceRate
from xerparser.model.classes.rsrcrcat import ResourceCat
from xerparser.model.classes.schedoption import SchedOption
from xerparser.model.classes.task import Task
from xerparser.model.classes.taskactv import TaskActv
from xerparser.model.classes.taskpred import TaskPred
from xerparser.model.classes.udftypes import UDFType
from xerparser.model.classes.udfvalue import UDFValue
from xerparser.model.classes.wbs import WBS
from xerparser.model.classes.currency import Currency
from xerparser.model.classes.taskrsrc import TaskRsrc
from xerparser.model.classes.roles import Role
from xerparser.model.classes.account import Account
from xerparser.model.classes.rolerate import RoleRate
from xerparser.model.tasks import Tasks
from xerparser.model.predecessors import Predecessors
from xerparser.model.projects import Projects
from xerparser.model.wbss import WBSs
from xerparser.model.resources import Resources
from xerparser.model.accounts import Accounts



import json

class Resource:
    obj_list = []

    def __init__(self, params):
        self.rsrc_id = int(params.get('rsrc_id').strip()) if params.get('rsrc_id') else None
        self.parent_rsrc_id = int(params.get('parent_rsrc_id').strip()) if params.get('parent_rsrc_id') else None
        self.clndr_id = int(params.get('clndr_id').strip()) if params.get('clndr_id') else None
        self.role_id = int(params.get('role_id').strip()) if params.get('role_id') else None
        self.shift_id = params.get('shift_id').strip() if params.get('shift_id') else None
        self.user_id = params.get('user_id').strip() if params.get('user_id') else None
        self.pobs_id = params.get('pobs_id').strip() if params.get('pobs_id') else None
        self.guid = params.get('guid').strip() if params.get('guid') else None
        self.rsrc_seq_num = params.get('rsrc_seq_num').strip() if params.get('rsrc_seq_num') else None
        self.email_addr = params.get('email_addr').strip() if params.get('email_addr') else None
        self.employee_code = params.get('employee_code').strip() if params.get('employee_code') else None
        self.office_phone = params.get('office_phone').strip() if params.get('office_phone') else None
        self.other_phone = params.get('other_phone').strip() if params.get('other_phone') else None
        self.rsrc_name = params.get('rsrc_name').strip() if params.get('rsrc_name') else None
        self.rsrc_short_name = params.get('rsrc_short_name').strip() if params.get('rsrc_short_name') else None
        self.rsrc_title_name = params.get('rsrc_title_name').strip() if params.get('rsrc_title_name') else None
        self.def_qty_per_hr = params.get('def_qty_per_hr').strip() if params.get('def_qty_per_hr') else None
        self.cost_qty_type = params.get('cost_qty_type').strip() if params.get('cost_qty_type') else None
        self.ot_factor = params.get('ot_factor').strip() if params.get('ot_factor') else None
        self.active_flag = params.get('active_flag').strip() if params.get('active_flag') else None
        self.auto_compute_act_flag = params.get('auto_compute_act_flag').strip() if params.get('auto_compute_act_flag') else None
        self.def_cost_qty_link_flag = params.get('def_cost_qty_link_flag').strip() if params.get('def_cost_qty_link_flag') else None
        self.ot_flag = params.get('ot_flag').strip() if params.get('ot_flag') else None
        self.curr_id = int(params.get('curr_id').strip()) if params.get('curr_id') else None
        self.unit_id = int(params.get('unit_id').strip()) if params.get('unit_id') else None
        self.rsrc_type = params.get('rsrc_type').strip() if params.get('rsrc_type') else None
        self.location_id = int(params.get('location_id').strip()) if params.get('location_id') else None
        self.rsrc_notes = params.get('rsrc_notes').strip() if params.get('rsrc_notes') else None
        self.load_tasks_flag = params.get('load_tasks_flag').strip() if params.get('load_tasks_flag') else None
        self.level_flag = params.get('level_flag').strip() if params.get('level_flag') else None
        self.last_checksum = params.get('level_flag').strip() if params.get('level_flag') else None
        Resource.obj_list.append(self)

    def get_id(self):
        return self.rsrc_id

    @classmethod
    def find_by_id(cls, id):
        obj = list(filter(lambda x: x.rsrc_id == id, cls.obj_list))
        if obj:
            return obj[0]
        return obj

    @property
    def parent(self):
        return self.parent_rsrc_id

    def __repr__(self):
        return self.rsrc_name

    def __str__(self):
        return self.rsrc_name

    def toJson(self):
        return json.dumps(self, default=lambda o: o.__dict__)


from xerparser.model.wbss import WBSs
from xerparser.model.tasks import Tasks

class Project:
    obj_list = []

    def __init__(self, params):

        self.proj_id = int(params.get('proj_id').strip()) if params.get('proj_id') else None
        self.fy_start_month_num = params.get('fy_start_month_num').strip() if params.get('fy_start_month_num') else None
        self.rsrc_self_add_flag = params.get('rsrc_self_add_flag').strip() if params.get('rsrc_self_add_flag') else None
        self.allow_complete_flag = params.get('allow_complete_flag').strip() if params.get('allow_complete_flag') else None
        self.rsrc_multi_assign_flag = params.get('rsrc_multi_assign_flag').strip() if params.get('rsrc_multi_assign_flag') else None
        self.checkout_flag = params.get('checkout_flag').strip() if params.get('checkout_flag') else None
        self.project_flag = params.get('project_flag').strip() if params.get('project_flag') else None
        self.step_complete_flag = params.get('step_complete_flag').strip() if params.get('step_complete_flag') else None
        self.cost_qty_recalc_flag = params.get('cost_qty_recalc_flag').strip() if params.get('cost_qty_recalc_flag') else None
        self.batch_sum_flag = params.get('batch_sum_flag').strip() if params.get('batch_sum_flag') else None
        self.name_sep_char = params.get('name_sep_char').strip() if params.get('name_sep_char') else None
        self.def_complete_pct_type = params.get('def_complete_pct_type').strip() if params.get('def_complete_pct_type') else None
        self.proj_short_name = params.get('proj_short_name').strip() if params.get('proj_short_name') else None
        self.acct_id = params.get('acct_id').strip() if params.get('acct_id') else None
        self.orig_proj_id = params.get('orig_proj_id').strip() if params.get('orig_proj_id') else None
        self.source_proj_id = params.get('source_proj_id').strip() if params.get('source_proj_id') else None
        self.base_type_id = params.get('base_type_id').strip() if params.get('base_type_id') else None
        self.clndr_id = params.get('clndr_id').strip() if params.get('clndr_id') else None
        self.sum_base_proj_id = params.get('sum_base_proj_id').strip() if params.get('sum_base_proj_id') else None
        self.task_code_base = params.get('task_code_base').strip() if params.get('task_code_base') else None
        self.task_code_step = params.get('task_code_step').strip() if params.get('task_code_step') else None
        self.priority_num = params.get('priority_num').strip() if params.get('priority_num') else None
        self.wbs_max_sum_level = params.get('wbs_max_sum_level').strip() if params.get('wbs_max_sum_level') else None
        self.strgy_priority_num = params.get('strgy_priority_num').strip() if params.get('strgy_priority_num') else None
        self.last_checksum = params.get('last_checksum').strip() if params.get('last_checksum') else None
        self.critical_drtn_hr_cnt = params.get('critical_drtn_hr_cnt').strip() if params.get('critical_drtn_hr_cnt') else None
        self.def_cost_per_qty = params.get('def_cost_per_qty').strip() if params.get('def_cost_per_qty') else None
        self.last_recalc_date = params.get('last_recalc_date').strip() if params.get('last_recalc_date') else None
        self.plan_start_date = params.get('plan_start_date').strip() if params.get('plan_start_date') else None
        self.plan_end_date = params.get('plan_end_date').strip() if params.get('plan_end_date') else None
        self.scd_end_date = params.get('scd_end_date').strip() if params.get('scd_end_date') else None
        self.add_date = params.get('add_date').strip() if params.get('add_date') else None
        self.last_tasksum_date = params.get('last_tasksum_date').strip() if params.get('last_tasksum_date') else None
        self.fcst_start_date = params.get('fcst_start_date').strip() if params.get('fcst_start_date') else None
        self.def_duration_type = params.get('def_duration_type').strip() if params.get('def_duration_type') else None
        self.task_code_prefix = params.get('task_code_prefix').strip() if params.get('task_code_prefix') else None
        self.guid = params.get('guid').strip() if params.get('guid') else None
        self.def_qty_type = params.get('def_qty_type').strip() if params.get('def_qty_type') else None
        self.add_by_name = params.get('add_by_name').strip() if params.get('add_by_name') else None
        self.web_local_root_path = params.get('web_local_root_path').strip() if params.get('web_local_root_path') else None
        self.proj_url = params.get('proj_url').strip() if params.get('proj_url') else None
        self.def_rate_type = params.get('def_rate_type').strip() if params.get('def_rate_type') else None
        self.add_act_remain_flag = params.get('add_act_remain_flag').strip() if params.get('add_act_remain_flag') else None
        self.act_this_per_link_flag = params.get('act_this_per_link_flag').strip() if params.get('act_this_per_link_flag') else None
        self.def_task_type = params.get('def_task_type').strip() if params.get('def_task_type') else None
        self.act_pct_link_flag = params.get('act_pct_link_flag').strip() if params.get('act_pct_link_flag') else None
        self.critical_path_type = params.get('critical_path_type').strip() if params.get('critical_path_type') else None
        self.task_code_prefix_flag = params.get('task_code_prefix_flag').strip() if params.get('task_code_prefix_flag') else None
        self.def_rollup_dates_flag = params.get('def_rollup_dates_flag').strip() if params.get('def_rollup_dates_flag') else None
        self.use_project_baseline_flag = params.get('use_project_baseline_flag').strip() if params.get('use_project_baseline_flag') else None
        self.rem_target_link_flag = params.get('rem_target_link_flag').strip() if params.get('rem_target_link_flag') else None
        self.reset_planned_flag = params.get('reset_planned_flag').strip() if params.get('reset_planned_flag') else None
        self.allow_neg_act_flag = params.get('allow_neg_act_flag').strip() if params.get('allow_neg_act_flag') else None
        self.sum_assign_level = params.get('sum_assign_level').strip() if params.get('sum_assign_level') else None
        self.last_fin_dates_id = params.get('last_fin_dates_id').strip() if params.get('last_fin_dates_id') else None
        self.last_baseline_update_date = params.get('last_fin_dates_id').strip() if params.get('last_fin_dates_id') else None
        self.cr_external_key = params.get('cr_external_key').strip() if params.get('cr_external_key') else None
        self.apply_actuals_date = params.get('apply_actuals_date').strip() if params.get('apply_actuals_date') else None
        self.location_id = params.get('location_id') if params.get('location_id') else None
        self.loaded_scope_level = params.get('loaded_scope_level').strip() if params.get('loaded_scope_level') else None
        self.export_flag = params.get('export_flag').strip() if params.get('export_flag') else None
        self.new_fin_dates_id = params.get('new_fin_dates_id').strip() if params.get('new_fin_dates_id') else None
        self.baselines_to_export = params.get('baselines_to_export').strip() if params.get('baselines_to_export') else None
        self.baseline_names_to_export = params.get('baseline_names_to_export').strip() if params.get('baseline_names_to_export') else None
        self.next_data_date = params.get('next_data_date').strip() if params.get('next_data_date') else None
        self.close_period_flag = params.get('close_period_flag').strip() if params.get('close_period_flag') else None
        self.sum_refresh_date = params.get('sum_refresh_date').strip() if params.get('sum_refresh_date') else None
        self.trsrcsum_loaded = params.get('trsrcsum_loaded').strip() if params.get('trsrcsum_loaded') else None
        Project.obj_list.append(self)

    @property
    def id(self):
        return self.proj_id


    @property
    def activities(self):
        return Tasks.get_by_project(self.proj_id)

    @property
    def wbss(self):
        return WBSs.get_by_project(self.proj_id)

    def __repr__(self):
        return self.proj_short_name


class Currency:
    obj_list = []

    def __init__(self, params):
        # Unique ID generated by the system.
        self.curr_id = int(params.get('curr_id')) if params.get('curr_id') else None
        # The number of decimal places displayed.
        self.decimal_digit_cnt = int(params.get('decimal_digit_cnt')) if params.get('decimal_digit_cnt') else None
        # The symbol used to identify each defined currency.
        self.curr_symbol = params.get('curr_symbol').strip() if params.get('curr_symbol') else None
        # The decimal symbol displayed.
        self.decimal_symbol = params.get('decimal_symbol').strip() if params.get('decimal_symbol') else None
        # The symbol used to group the numbers.
        self.digit_group_symbol = params.get('digit_group_symbol').strip() if params.get('digit_group_symbol') else None
        # The symbol used to display a positive currency.
        self.pos_curr_fmt_type = params.get('pos_curr_fmt_type').strip() if params.get('pos_curr_fmt_type') else None
        # The symbol used to display a negative currency.
        self.neg_curr_fmt_type = params.get('neg_curr_fmt_type').strip() if params.get('neg_curr_fmt_type') else None
        # The names of all defined currencies.
        self.curr_type = params.get('curr_type').strip()
        # The identifiers for all currencies defined in Project Management.
        self.curr_short_name = params.get('curr_short_name').strip() if params.get('curr_short_name') else None
        # Currency Group Digit Count
        self.group_digit_cnt = params.get('group_digit_cnt').strip()
        # The current exchange rate between the selected currency and the base currency.
        self.base_exch_rate = params.get('base_exch_rate').strip() if params.get('base_exch_rate') else None

        Currency.obj_list.append(self)

    def get_id(self):
        return self.curr_id


    @classmethod
    def find_by_id(cls, id):
        obj = list(filter(lambda x: x.curr_id == id, cls.obj_list))
        if obj:
            return obj[0]
        return obj

    def __repr__(self):
        return self.curr_symbol + ' ' + self.curr_type


from setuptools import setup

setup(
    name='PyP6Xer',
    version='1.011.01',
    packages=['xerparser', 'xerparser.model', 'xerparser.model.classes'],
    url='',
    license='GNU GENERAL PUBLIC LICENSE',
    author='Hassan',
    author_email='hassan.emam@hotmail.com',
    description='Parser for XER written in Python'
)



from xerparser.model.classes.project import Project

class Projects:
    _projects = []

    def __init__(self):
        self.index =0

    def add_project(self, params):
        prj = Project(params)
        self._projects.append(prj)

    def find_by_id(self, id):
        obj = list(filter(lambda x: x.proj_id == id, Project.obj_list))
        if obj:
            return obj[0]
        return obj

    def __iter__(self):
        return self

    def __next__(self):
        if self.index >= len(self._projects):
            raise StopIteration
        idx = self.index
        self.index +=1
        return self._projects[idx]